// NEWBUFF, a redone implementation of InstaOg using SkunkWorks.
// Copyright (C) 2002 Ron Kuris <rk@divdb.com>
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// InstaOg simply recasts the same sequence of spells.  This macro decides
// what the BEST spell is based on a scoring system.  Spells that are in
// effect already will be scored lower, hoping for other, better spells to
// be cast.
// Works with mana pool, just like InstaOg, of course.
// Automatically starts casting harder spells once you get the skill to do so.

// REMEMBER TO SELECT AN OBJECT before you start for your ITEM spells
// otherwise it runs in test mode

// TODO list:
// Learn new spells by reading scrolls if you don't know them already
// Respond to /tells for portals, getting some tapers and comps from the public
// Compute burn rate for tapers?
// Better reporting

// Configuration section
var fudge = 20;			// cast spells with diff 10 pts above skill
				// setting this HIGHER will make it more
				// aggressive, at the expense of extra tapers
				// For minimal burn rate, set lower (zero is ok)

var gotMatch;
var healingon = 1;		// set to zero to disable healing with kits
				// healing is only done once every 2 minutes
				// to gain experience

// SPELL CONFIGURATION
// Each possible spell to cast looks like this:
// new Spell(0, 1, 1177, 25, "Revitalize Self I", "revit"),
//           ^ School (0=life, 1=creature, 2=item)
//              ^ Spell level (1-7)
//                 ^^^^ Spell ID
//                       ^^ Spell difficulty
//                            ^^^^^^^^^^^^^^^^^^ Spell Name
//                                 Calculation Fn ^^^^^^

// If you do not know the self-vuln spells (you had to get them before the
// July 2002 patch), then you may comment them out, although the system will
// figure it out automatically if you don't do that.
var aSpells = new Array(

new Spell(0, 1, 1177, 25, "Revitalize Self I", "revit"),
new Spell(0, 1, 1278, 25, "Health to Mana Self I", "h2m"),
new Spell(0, 1, 212, 25, "Mana Renewal Self I", "renew"),
new Spell(0, 1, 1676, 25, "Stamina to Mana Self I", "s2m"),
new Spell(0, 1, 54, 25, "Rejuvenation Self I", "rejuv"),

new Spell(0, 2, 1178, 50, "Revitalize Self II", "revit"),
new Spell(0, 2, 1279, 50, "Health to Mana Self II", "h2m"),
new Spell(0, 2, 213, 50, "Mana Renewal Self II", "renew"),
new Spell(0, 2, 1677, 50, "Stamina to Mana Self II", "s2m"),
new Spell(0, 2, 189, 50, "Rejuvenation Self II", "rejuv"),

new Spell(0, 2, 528, 75, "Acid Vulnerability Self II", "norm"),

new Spell(0, 3, 1179, 100, "Revitalize Self III", "revit"),
new Spell(0, 3, 1280, 100, "Health to Mana Self III", "h2m"),
new Spell(0, 3, 214, 100, "Mana Renewal Self III", "renew"),
new Spell(0, 3, 1678, 100, "Stamina to Mana Self III", "s2m"),
new Spell(0, 3, 190, 100, "Rejuvenation Self III", "rejuv"),

new Spell(0, 3, 529, 125, "Acid Vulnerability Self III", "norm"),

new Spell(0, 4, 1180, 150, "Revitalize Self IV", "revit"),
new Spell(0, 4, 1679, 150, "Stamina to Mana Self IV", "s2m"),
new Spell(0, 4, 215, 150, "Mana Renewal Self IV", "renew"),
new Spell(0, 4, 1702, 150, "Health to Mana Self IV", "h2m"),
new Spell(0, 4, 191, 150, "Rejuvenation Self IV", "rejuv"),

// new Spell(0, 4, 530, 175, "Acid Vulnerability Self IV", "norm"),

new Spell(0, 5, 1181, 200, "Revitalize Self V", "revit"),
new Spell(0, 5, 1680, 200, "Stamina to Mana Self V", "s2m"),
new Spell(0, 5, 1703, 200, "Health to Mana Self V", "h2m"),
new Spell(0, 5, 169, 200, "Regeneration Self V", "regen"),
new Spell(0, 5, 192, 200, "Rejuvenation Self V", "rejuv"),
new Spell(0, 5, 216, 200, "Mana Renewal Self V", "renew"),

// new Spell(0, 5, 531, 225, "Acid Vulnerability Self IV", "norm"),

new Spell(0, 6, 1182, 250, "Revitalize Self VI", "revit"),
new Spell(0, 6, 1681, 250, "Stamina to Mana Self VI", "s2m"),
new Spell(0, 6, 170, 250, "Regeneration Self VI", "regen"),
new Spell(0, 6, 193, 250, "Rejuvenation Self VI", "rejuv"),
new Spell(0, 6, 1704, 250, "Health to Mana Self VI", "h2m"),
new Spell(0, 6, 217, 250, "Mana Renewal Self VI", "renew"),

new Spell(1, 2, 899, 25, "Leadership Mastery Self II", "norm"),

new Spell(1, 2, 1350, 50, "Endurance Self II", "endurance"),
new Spell(1, 2, 1422, 50, "Focus Self II", "focus"),
new Spell(1, 2, 1446, 50, "Willpower Self II", "will"),
new Spell(1, 2, 558, 50, "Creature Mastery Self II", "crit"),
new Spell(1, 2, 582, 50, "Item Mastery Self II", "item"),
new Spell(1, 2, 606, 50, "Life Mastery Self II", "life"),
new Spell(1, 2, 654, 50, "Mana Conversion Self II", "conv"),

new Spell(1, 3, 900, 75, "Leadership Mastery Self III", "norm"),

new Spell(1, 3, 1351, 100, "Endurance Self III", "endurance"),
new Spell(1, 3, 1423, 100, "Focus Self III", "focus"),
new Spell(1, 3, 1447, 100, "Willpower Self III", "will"),
new Spell(1, 3, 559, 100, "Creature Mastery Self III", "crit"),
new Spell(1, 3, 583, 100, "Item Mastery Self III", "item"),
new Spell(1, 3, 607, 100, "Life Mastery Self III", "life"),
new Spell(1, 3, 655, 100, "Mana Conversion Self III", "conv"),

new Spell(1, 4, 901, 125, "Leadership Mastery Self IV", "2*norm"),

new Spell(1, 4, 1352, 150, "Endurance Self IV", "endurance"),
new Spell(1, 4, 1424, 150, "Focus Self IV", "focus"),
new Spell(1, 4, 1448, 150, "Willpower Self IV", "will"),
new Spell(1, 4, 560, 150, "Creature Mastery Self IV", "crit"),
new Spell(1, 4, 584, 150, "Item Mastery Self IV", "item"),
new Spell(1, 4, 608, 150, "Life Mastery Self IV", "life"),
new Spell(1, 4, 656, 150, "Mana Conversion Self IV", "conv"),
new Spell(1, 4, 949, 150, "Fealty Self IV", "fealty"),

new Spell(1, 5, 1353, 200, "Endurance Self V", "endurance"),
new Spell(1, 5, 1425, 200, "Focus Self V", "focus"),
new Spell(1, 5, 1449, 200, "Willpower Self V", "will"),
new Spell(1, 5, 561, 200, "Creature Mastery Self V", "crit"),
new Spell(1, 5, 585, 200, "Item Mastery Self V", "item"),
new Spell(1, 5, 609, 200, "Life Mastery Self V", "life"),
new Spell(1, 5, 657, 200, "Mana Conversion Mastery Self V", "conv"),
new Spell(1, 5, 950, 200, "Fealty Self V", "fealty"),

new Spell(1, 6, 1354, 250, "Endurance Self VI", "endurance"),
new Spell(1, 6, 1426, 250, "Focus Self VI", "focus"),
new Spell(1, 6, 1450, 250, "Willpower Self VI", "will"),
new Spell(1, 6, 562, 250, "Creature Mastery Self VI", "crit"),
new Spell(1, 6, 586, 250, "Item Enchantment Mastery Self VI", "item"),
new Spell(1, 6, 610, 250, "Life Mastery Self VI", "life"),
new Spell(1, 6, 658, 250, "Mana Conversion Mastery Self VI", "conv"),
new Spell(1, 6, 951, 250, "Fealty Self VI", "fealty"),

new Spell(2, 1, 35, 25, "Blood Drinker I", "norm"),
new Spell(2, 2, 1470, 50, "Hide Value II", "norm"), // sorry, I don't have BD2
new Spell(2, 2, 1612, 50, "Blood Drinker II", "norm"),
new Spell(2, 3, 1613, 100,"Blood Drinker III", "norm"),
new Spell(2, 4, 1614, 150, "Blood Drinker IV", "norm"),
new Spell(2, 5, 1615, 200, "Blood Drinker V", "norm"),
new Spell(2, 6, 1616, 250, "Blood Drinker VI", "norm")
);

// END OF CONFIGURATION SECTION; you shouldn't need to touch anything else

// Internal timers
var tHealTimer;			// Healing timer
var tMasterTimer;		// master timer

// Constants
var tEasyTimer = 135000;	//  ms for easy spell only (2:15)
var tResetTimer = 135000;	//  ms to reset levels (2:15)
var tForceReset = 165000;	// forced reset after 2:45

var objSelected;		// object selected at start

var aTimers = new Array();	// timers for each school
var aDiffs = new Array(0, 0, 0);// last spell difficulty cast
var aBestDiff = new Array();	// best difficulty we could find per school

var healthMax = 150;		// current maximum health
var staminaMax = 150;		// current maximum stamina
var manaMax = 150;		// current maximum mana

var healthCur;			// current health
var staminaCur;			// current stamina
var manaCur;			// current mana

var startExp;			// starting experience
var defout;			// default out

var aSkills = new Array(150, 150, 150);
var schoolPause = new Array(0, 0, 0);
var timeLeft;
var lyingdown = 0;

// creation function for spells
function Spell(school, level, id, diff, name, scorefn)
    {
    this.school = school;
    this.level = level;
    this.id = id;
    this.diff = diff;
    this.name = name;
    this.scorefn = scorefn;
    }

function revit(spell)		// revitalize self scoring; scores very high
    {
    var stamDiff = staminaMax - staminaCur;
    if ( spell.level == 1 && stamDiff >= 20 )
    	return 1000;		// very useful
    if ( spell.level == 2 && stamDiff >= 30 )
    	return 2000;		// very useful
    if ( spell.level == 3 && stamDiff >= 50 )
    	return 3000;		// very useful
    if ( spell.level == 4 && stamDiff >= 75 )
    	return 4000;		// very useful
    if ( spell.level == 5 && stamDiff >= 100 )
    	return 5000;		// very useful
    if ( spell.level == 6 && stamDiff >= 150 )
    	return 6000;		// very useful
    return (stamDiff + 1)/10;	// return .1 even if no mana restores
    }
function norm(spell)		// normal spell scoring
    {
    return 1;
    }
function rejuv(spell)		// rejuvinate self scoring
    {
    return regenlike(spell, 0x5008, 4, 10000); // 1000 - 2000 pts
    }
function regen(spell)		// regeneration self scoring
    {
    var res = regenlike(spell, 0x5008, 3, 2000); // 100 - 200 pts
    if ( spell.level == 6 )
	skapi.OutputLine("regen" + spell.level + " scores " + res, defout);
    return res;
    }
function regenlike(spell, flags, attr, fact)
    {
    var bonuses = currentBonus(flags, attr);
    var newlvl = 0;
    switch (spell.level)
    {
    	case 1:	newlvl = 1.1; break;
    	case 2:	newlvl = 1.25; break;
    	case 3:	newlvl = 1.4; break;
    	case 4:	newlvl = 1.55; break;
    	case 5:	newlvl = 1.7; break;
    	case 6:	newlvl = 1.85; break;
	case 7: newlvl = 2; break;
    }
    if ( bonuses < newlvl - .01 )
    	return fact*(newlvl - bonuses);
    return 0.5;
    }
function endurance(spell)	// endurance self scoring
    {
    return .9*attrlike(spell, 0x9001, attrEndurance);
    }
function attrlike(spell, flags, attr)
    {
    var bonuses = currentBonus(flags, attr);
    var wouldGet = 5+(5*spell.level) - bonuses;
//    skapi.OutputLine("wouldGet is " + wouldGet, opmConsole);

    if ( wouldGet <= 0 )
    	{
	return 0.5;		// not a useful spell
	}
    return 100*wouldGet;
    }
function currentBonus(flag, value)	// compute current bonus of attribute
    {
    var e = new Enumerator(skapi.cospell);
    var activespell;
    var bestBonus = 0;
    for (; !e.atEnd(); e.moveNext())
	{
	activespell = e.item();
	if ( (activespell.flags == flag)
	&&   (activespell.avsAffected == value)
	&&   (activespell.csecRemain == -1 || activespell.csecRemain > 60)
	&&   (activespell.dlvl > bestBonus) )
		{
		if ( activespell.csecRemain != -1 && activespell.csecRemain < 60*5 )
			bestbonus = activespell.dlvl * (activespell.csecRemain/(60*5));
		else
			bestBonus = activespell.dlvl;
		timeLeft = activespell.csecRemain;
		}
	}
//    skapi.OutputLine("Current bonus is " + bestBonus, opmConsole);
    return bestBonus;
    }
function item(spell)
    {
    return 1.5*attrlike(spell, 0x9010, skidItemEnchantment);
    }
function life(spell)
    {
    return 2*attrlike(spell, 0x9010, skidLifeMagic);
    }
function conv(spell)
    {
    return 2.5*attrlike(spell, 0x9010, skidManaConversion);
    }
function crit(spell)
    {
    return 3*attrlike(spell, 0x9010, skidCreatureEnchantment);
    }
function s2m(spell)
    {
    var rval;
    if ( staminaCur < staminaMax*.9 || manaMax - manaCur < staminaCur/2 )
    	{
	// this is not a great spell; stamina is at not at 90% of max
	// or we do not need the mana right now
	if ( aBestDiff[0] == spell.diff )	// is this the next diff?
	    rval = 1;				// still reasonable to cast it
//	skapi.OutputLine('no xp, no gain' + staminaCur + "/" + staminaMax, defout);
	rval = 0;				// no exp, no gain
	}
    // we have the stamina to spare, and need the mana
    else if ( aBestDiff[0] == spell.diff )
	{
	rval = 100*spell.level;
	}
    else
	{
        if ( spell.diff > aSkills[spell.school] )  // no fudge
	    rval = 0;				// don't cast: too risky
    	else
	    rval = 10*spell.level;		// could use the mana
	}
//    skapi.OutputLine(
//	"Score for s2m " + spell.level + "=" + rval + " " + manaCur + "/" + manaMax,
 //  	defout);
    return rval;
    }
function h2m(spell)
    {
    if ( healthCur < healthMax*.9 || manaMax - manaCur < healthCur / 2 )
    	{
//	if ( aBestDiff[0] == spell.diff )	// next difficulty?
//	    return .5;				// reasonable to cast
	return 0;				// no exp, not enough gain
	}
    // we have the health to spare, and need the mana
    if ( aBestDiff[0] == spell.diff )
        return 100*spell.level;			// perfect match
    else
	{
        if ( spell.diff > aSkills[spell.school] )  // no fudge
	    return 0;				// don't cast: too risky
    	return 10*spell.level;			// could use the mana
	}
    }
function focus(spell)
{
    return attrlike(spell, 0x9001, attrFocus);
}
function will(spell)
{
    return attrlike(spell, 0x9001, attrSelf);
}
function fealty(spell)
{
    return attrlike(spell, 0x9010, skidLoyalty);
}
function renew(spell)
{
    return regenlike(spell, 0x5008, 5, 1000); // 100 - 200 pts
    // Spell 20488/5/1.5499999523162841/215/1693
}

function main()
    {
    objSelected = skapi.acoSelected;
    var handler = new CommandHandler();
    skapi.AddHandler(evidOnChatServer, handler);
    skapi.AddHandler(evidOnStatSkillExp, handler);
    skapi.AddHandler(evidOnTell, handler);
    var Timeout = skapi.TimerNew();
    if ( !objSelected )
    	{
        skapi.OutputLine(
		"[NewBuff] Test mode; no object selected",
		opmConsole);
	manaCur = 150;
	healthCur = 150;
	staminaCur = 150;
	defout = opmConsole;
	}
    else
	{
        skapi.OutputLine(
    	"[NewBuff] Starting; casting on " + objSelected.szName,
    	opmConsole);
	defout = opmChatWnd;
	}
    skapi.WaitEvent(10, wemFullTimeout);	// clear all pending events

    tMasterTimer = skapi.TimerNew();
    aTimers[0] = skapi.TimerNew();
    aTimers[1] = skapi.TimerNew();
    aTimers[2] = skapi.TimerNew();
    aTimers[0].cmsec = tResetTimer;
    aTimers[1].cmsec = tResetTimer;
    aTimers[2].cmsec = tResetTimer;
    tHealTimer = skapi.TimerNew();
    tHealTimer.cmsec = tResetTimer;
    startExp = skapi.expTotal;

    while (1)				// main loop
        {
	var bestScore = 0;		// score of the best spell
	var bestSpell;			// best spell to cast
	var i;				// temp loop var

	if ( objSelected )
	    {
	    aSkills[0] = skapi.SkinfoFromSkid(skidLifeMagic).lvlCur;
	    aSkills[1] = skapi.SkinfoFromSkid(skidCreatureEnchantment).lvlCur;
	    aSkills[2] = skapi.SkinfoFromSkid(skidItemEnchantment).lvlCur;
	    }
	if ( objSelected )
	    manaCur = skapi.manaCur;
//	skapi.OutputLine("[NewBuff] L/C/I "+aSkills[0]+"/"+aSkills[1]+"/"+aSkills[2] + " " + aTimers[0].cmsec + "/" + aTimers[1].cmsec + "/" + aTimers[2].cmsec + " H/M/S " + healthCur + "/" + manaCur + "/" + staminaCur,defout);
	// first reset all the school timers if necessary
	for ( i = 0; i < 3; i++ )
		{
		if ( aTimers[i].cmsec > tEasyTimer )
			{
			aDiffs[i] = 0;		// reset difficulty
			}
		}

	// figure out the difficulty needed for each school
	aBestDiff[0] = 999;
	aBestDiff[1] = 999;
	aBestDiff[2] = 999;
	for ( i = 0; i < aSpells.length; i++)
		{
		if ( "blacklisted" in aSpells[i] )
			continue;
		if ( aSpells[i].diff > aDiffs[aSpells[i].school]
		&&   aSpells[i].diff < aBestDiff[aSpells[i].school] )
			aBestDiff[aSpells[i].school] = aSpells[i].diff;
		}

	// figure out which spell to cast
	if ( objSelected )
		{
		healthMax = skapi.SkinfoFromVital(vitalHealthMax).lvlCur;
		staminaMax = skapi.SkinfoFromVital(vitalStaminaMax).lvlCur;
		manaMax = skapi.SkinfoFromVital(vitalManaMax).lvlCur;
		healthCur = skapi.healthCur;
		staminaCur = skapi.staminaCur;
		manaCur = skapi.manaCur;
		}

	for ( i = 0; i < aSpells.length; i++)
	    {
	    var curSpell = aSpells[i];
	    var curScore = 0;
	    // skip if spell is blacklisted ("you don't know it")
	    if ( "blacklisted" in curSpell )
		    continue;

	    // skip if spell is too difficult to cast at current skill
	    if ( curSpell.diff - fudge > aSkills[curSpell.school] )
		    continue;

	    // skip if you don't have enough mana to cast the spell
	    if ( manaCur < curSpell.level * 10 )
		    continue;

	    if ( curSpell.scorefn != "s2m" && curSpell.scorefn != "h2m" )
	    	{
	        if ( schoolPause[curSpell.school] > tMasterTimer.cmsec )
	    	    continue;
		}

	    // check for difficulty we want
	    if ( (curSpell.diff != aBestDiff[curSpell.school])
	    &&   ((curSpell.diff > aBestDiff[curSpell.school])
	    ||   (curSpell.scorefn != "s2m" && curSpell.scorefn != "h2m")) )
		    continue;

	    eval("curScore = " + curSpell.scorefn + "(curSpell)");
//          skapi.OutputLine(curSpell.name + " got " + curScore, opmConsole);
//	    curScore *= schoolPenalties[curSpell.school];
	    if ( curScore > bestScore )
		{
		bestSpell = curSpell;
		bestScore = curScore;
//		skapi.OutputLine("Best now "+ curSpell.name + " (" + bestScore + ")", opmConsole);
		}
	    }
	var timeran = tMasterTimer.cmsec;
	if ( bestScore > 0 )
	    {
	    curSpell = bestSpell;
	    // cast the spell; make sure you're in combat mode
	    skapi.OutputLine("Casting " + curSpell.name + "(" + bestScore + ", Time=" + totime(timeran) + ", SchoolTime=" + totime(aTimers[curSpell.school].cmsec) + ")", defout);
	    while ( !skapi.fCombatMode && objSelected )
		{
		skapi.Keys("{ToggleCombat}");
		skapi.WaitEvent(2000 + lyingdown*2000, wemSpecific, evidOnCombatMode);
		lyingdown = 0;
		}
	    skapi.WaitEvent(10);	// clear all pending events
	    var oldxp = skapi.expTotal;	// save experience points
	    if ( curSpell.school == 2 )	/* item needs target */
		    skapi.CastSpell(curSpell.id, objSelected);
	    else
		    skapi.CastSpell(curSpell.id);

	    if ( !objSelected )
		{
		manaCur = manaCur - curSpell.level*10;
		if ( curSpell.scorefn == "h2m" || curSpell.scorefn == "s2m" )
		    manaCur = manaMax;
		}

	    Timeout.cmsec = 0;
	    gotMatch = false;
	    while ( Timeout.cmsec < 1500*curSpell.level )
	    	{
		var curEvt = skapi.WaitEvent(1500*curSpell.level - Timeout.cmsec, wemSingle);
		if ( curEvt == evidOnSpellCastSelf )
			break;
		if ( curEvt == evidOnChatServer && gotMatch )
			break;
		if ( curEvt == evidOnSpellFailSelf )
			break;
		}
	    var skarc = skapi.arc;
	    if ( Timeout.cmsec >= 1500*curSpell.level )
		    {
		    skarc = arcTooBusy;
		    skapi.OutputLine("Timed out", defout);
		    }
	    	
	    switch (skarc)
		    {
		    case arcSuccess:
			    if ( aDiffs[curSpell.school] < curSpell.diff )
				    {
				    skapi.WaitEvent(300, wemSpecific, evidOnStatSkillExp);
				    aDiffs[curSpell.school] = curSpell.diff;
			    	    }
			    else
				    skapi.WaitEvent(100, wemFullTimeout);
			    if ( aTimers[curSpell.school].cmsec > tResetTimer )
	    			schoolPause[curSpell.school] = tMasterTimer.cmsec + 8000;
			    if ( aTimers[curSpell.school].cmsec > tForceReset )
			    	statmsg(curSpell.school);
			    break;
		    case arcDontKnowSpell:
			    skapi.OutputLine("No longer casting " + curSpell.name, defout);
			    curSpell.blacklisted = true;
			    break;
		    case arcOutOfComps:
			    skapi.OutputLine("Out of comps!", opmConsole);
			    if ( skapi.CitemOfKindInInv("Prismatic Taper") < 10 )
	    			skapi.Keys("~{Esc}");
			    // probably out of a scarab; stop casting this spell
			    curSpell.blacklisted = true;
			    break;
		    case arcTooBusy:
	    		    skapi.WaitEvent(100); // clear all pending events
		    	    break;
		    case arcFizzle:
			    break;
		    default:
			    skapi.OutputLine("Strange result " + skarc, defout);
			    break;
		    }
	    }
	else
	    {
	    while ( skapi.fCombatMode )
		{
		skapi.Keys("{ToggleCombat}");
		skapi.WaitEvent(1500, wemFullTimeout);
		}
	    if ( healme() == 0 && (staminaCur < staminaMax*.9 || manaCur < manaMax*.9) )
	    	{
	    	skapi.Keys("b");	// lie down
		lyingdown = 1;
	        skapi.WaitEvent(2000,wemFullTimeout); // check again in 2s
		}
	    }
	}
    }
function CommandHandler()
    {
    this.OnChatServer = OnChatServer;
    this.OnStatSkillExp = OnStatSkillExp;
    this.OnTell = OnTell;
    }
function OnTell(acoSender, acoReceiver, szMsg)
    {
    // Use acoSender.szName</b></a> to get the sender's name
    if ( szMsg == "progress" )
    	{
	var gotxp = skapi.expTotal - startExp;
	skapi.Keys("{Return}/tell " + acoSender.szName + ", I'm running a macro and have gotten " + gotxp + " xp so far this run (" + skapi.DexpToLvl() + " left to get to level " + skapi.lvl + 1  + "){Return}");
	}
    else
	skapi.Keys("{Return}/tell " + acoSender.szName + ", I'm not here right now (inactive for " + totime(tMasterTimer.cmsec) + ").  Tell me 'progress' for more info." + "{Return}");
    }
function OnStatSkillExp(skid, szSkill, diff, expTotal, dsklvl)
{
   switch (skid)
   {
   case skidCreatureEnchantment:
   	if ( aTimers[1].cmsec > tResetTimer )
		{
		statmsg(1);
		}
	break;
   case skidItemEnchantment:
   	if ( aTimers[2].cmsec > tResetTimer )
		{
		statmsg(2);
		}
	break;
   case skidLifeMagic:
   	if ( aTimers[0].cmsec > tResetTimer )
		{
		statmsg(0);
		}
	break;
   case skidManaConversion:
   	break;
   default:
        skapi.OutputLine(szSkill + " gain (skid=" + skid + ", diff=" + diff + ")", defout);
	break;
   }
}
function statmsg(sch)
{
	var smsg = "Sch" + sch + " reset at " + totime(aTimers[sch].cmsec);
	var gotxp = skapi.expTotal - startExp;
	smsg += "; " + skapi.DexpToLvl() + " to level";
	smsg += "; XP/hr: " + parseInt(gotxp/(tMasterTimer.cmsec/3600000.0)) + ", To level: ";
	if ( gotxp > 0 )
	   {
	   smsg += totime(skapi.DexpToLvl()/gotxp*tMasterTimer.cmsec);
	   }
	skapi.OutputLine(smsg, opmConsole);
	skapi.OutputLine(smsg, opmChatWnd);
	aTimers[sch].cmsec = 0;
}
function healme()
	{
//	skapi.OutputLine("Health check: " + skapi.healthCur + "/" + healthMax);
	if ( !healingon )
		return 0;
	if ( skapi.healthCur != skapi.healthMax )
		if ( tHealTimer.cmsec > 130000 )
		{
		// healing needed
//		skapi.OutputLine("Healing Needed",opmConsole);
		var kit = skapi.AcoFindInInv("Treated Healing Kit");
		if ( !kit ) kit = skapi.AcoFindInInv("Good Healing Kit");
		if ( !kit ) kit = skapi.AcoFindInInv("Excellent Healing Kit");
		if ( !kit ) kit = skapi.AcoFindInInv("Peerless Healing Kit");
		if ( kit )
			{
			kit.UseOnAco(skapi.acoChar);
			skapi.WaitEvent(2500, wemFullTimeout);
			tHealTimer.cmsec = 0;
			return 1;
			// reset timer if we healed to full health??
			// if ( skapi.healthCur == skapi.SkinfoFromVital(vitalHealthMax).lvlCur)
			}
		else
			// no kits right now; reset timer
			tHealTimer.cmsec = 0;
		}
	return 0;
	}
function totime(tm)
    {
    var res = "";
    tm = parseInt(tm);
    if ( tm > 60*60*1000)
    	{
	res = parseInt(tm/3600000) + ":";
	tm -= 3600000 * parseInt(tm/3600000);
	}
    var p1 = parseInt(tm/60000)+100 + "";
    res = res + p1.substr(1,2) + ":";
    tm -= parseInt(tm/60000)*60000;
    p1 = parseInt(tm/1000)+100 + "";
    res = res + p1.substr(1,2) + "." + parseInt((tm%1000)/100);
    return res;
    }
function OnChatServer(text, cmc)
{
    if ( (text.match(/You cast/)) || (text.match(/You gain/)) )
    	gotMatch = true;
}
// buffin buffed??
// Spell 20488(0x5008)/69/0.5699999928474426/518/1095 Acid Prot IV
// Spell 20488/65/0.5/1137/1087 Pierce V
// Spell 20488(0x5008)/4/1.850000023841858/193/1968 Rejuv Self VI
// Spell 20488/3/1.7000000476837158/163/639 Regen Other V
// Spell 20488/4/1.850000023841858/188/1535 Rejuv Other VI
// Spell 36880(0x9010)/21/30/878/1227 Healing
// Spell 36880/2/30/471/1180 Bow mastery
// Spell 36880/6/30/248/1174 Invul
// Spell 36865(0x9001)/3/30/1401/1165 Quickness
// Spell 36865/4/30/1377/1157 Coord V
// Spell 41088/0/150/1311/1080 Armor
// Spell 36865/1/30/1336/735 Str
// Spell 36865/3/35/1408/1631 Quickness
// Spell 36865(0x9001)/2/35/1360/1615 Endurance Other VI
// Spell 36865/2/30/1353/704 Endurance Self V
// Spell 36880/16/35/664/1566 Mana Conv Mastery Other VI
// Spell 36880/14/35/689/1561 Arcane
// Spell 36880/34/35/640/1549 War Magic Mastery
// Spell 36880/6/35/244/1544 Invul
// Spell 36880/32/30/585/522 Item Self V
// Spell 36880/16/30/657/504 Mana Conv Mastery Self V
// Spell 36880/33/30/609/500 Life Self V
// Spell 36865/6/30/1449/478 Willpower Self V
// Spell 36880/31/30/561/473 Creature Enchant Mastery Self V
// Spell 36865/5/30/1425/461 Focus Self V
// Spell 36865/6/25/1448/457 Willpower Self IV
// Spell 36865/5/25/1424/445 Focus Self IV
// Spell 36880/28/30/778/-1 Weapon Expertise Self V

// buffin
//Spell 36880/35/15/916
//Spell 36880/35/15/648
//Spell 36880/35/15/643
//Spell 36880/35/20/490
//Spell 36880/35/15/484
//Spell 36880/35/15/476
//Spell 36880/35/15/266
//Spell 36880/35/15/180
//Spell 36880/35/20/75
//Spell 36880/35/15/63
//Spell 36880/35/15/58
//Spell 36880/28/30/-1

// Tink
//Spell 36880/35/40/1375
//Spell 36880/36/40/1371
//Spell 36865/6/40/1366
//Spell 36865/1/35/340
//Spell 36865/4/35/306
//Spell 36865/5/40/1190
//Spell 36880/28/40/1134
//Spell 36865/5/10/1408
//Spell 36880/6/10/1412

